#include <sps2lib.h>
#include <sps2tags.h>
#include <sps2util.h>
#include <sps2regstructs.h>
#include <assert.h>
#include <memory.h>
#include <signal.h>
#include "PS2Defines.h"
#include "ps2maths.h"
#include "ps2matrix4x4.h"
#include "pad.h"
#include "sps2wrap.h"
#include "dma.h"


//number of boxes to draw
#define NUMBOX	(4)

// A structure type to hold box info
// You could make a PS2Box class if you want!
typedef struct
{
	int x, y;				//Bottom LH corner
	int z;					//z depth
	int w, h;				//width and height;
	unsigned char r, g, b;	//colour
}Ps2Box_t;

// Declare the boxes (Global for simplicity just now!)
Ps2Box_t Box[NUMBOX];

// Function prototypes
void GameLogic(void);
void BuildDMAPacket(void);
void ConfigureBox(Ps2Box_t * B, int x, int y, int z, int w, int h, 
								u_char r, u_char g, u_char b);
bool g_bLoop = true;

void sig_handle(int sig)
{
	g_bLoop = false;
}


int main()
{
	// Set the default parameters for the boxes to be drawn	
  	ConfigureBox(&Box[0], -200, -200, 1000, 100, 20, 128,   0,   0);
	ConfigureBox(&Box[1],  170, -200, 1000, 100, 20,   0, 128,   0);
	ConfigureBox(&Box[2], -200,  200, 1000, 100, 20,   0,   0, 128);
	ConfigureBox(&Box[3],  170,  200, 1000, 100, 20,   0, 128, 128);
	
	// Initialise control pad 0
	if(!pad_init(PAD_0, PAD_INIT_DIGITAL))
	{
		printf("Failed to initialise control pad\n");
		pad_cleanup(PAD_0);
		exit(0);
	}	

	SPS2Manager.Initialise(4096);	// 4096 * 4K Pages = 16MB Total

	VIFStaticDMA.Initialise(1536);	// 1536 * 4K Pages = 6MB Static DMA
	VIFDynamicDMA.Initialise(256);	// 256 * 4K Pages * 2 Buffers =
									// 2MB Dynamic DMA

	// Register our signal function for every possible signal (i.e. ctrl + c)
	for(int sig=0; sig<128; sig++)
		signal(sig, sig_handle);

	// Set up the DMA packet to clear the screen.
	SPS2Manager.InitScreenClear(0x20, 0, 0x20);

	while(g_bLoop)
	{
		// Fire off the previous frame's buffer (or init data if it's the first time)
		VIFDynamicDMA.Fire();

		pad_update(PAD_0);
		
		// Perform some game logic
  		GameLogic();

		// Clear the screen
		SPS2Manager.BeginScene();
		
		// Send data to GS
		BuildDMAPacket();
			
		// Swap the buffers
		SPS2Manager.EndScene();
		
		// Check for exit condition
		if((pad[0].buttons & PAD_START)&&(pad[0].buttons & PAD_SELECT)) g_bLoop = false;
	}
	
	// Cleanup the Pads
	pad_cleanup(PAD_0);
	
	return 0;
}

// Build the DMA Packet
void BuildDMAPacket(void)
{
	// Start DIRECT mode transfer
	VIFDynamicDMA.StartDirect();
	
	// Add the GIFTag
	VIFDynamicDMA.Add128(GS_GIFTAG_BATCH(	NUMBOX * 2,			// loop through vertices (NLOOP)
 											1,											//EOP
 											1,											//PRE
 											0x0E,				// 6 = Sprite | 8 = gourard shading (PRIM)
 											GIF_FLG_PACKED,								//FLG
 											GS_BATCH_2(GIF_REG_RGBAQ, GIF_REG_XYZ2)));	//BATCH					
	
	// Loop through all vertices
  	for (int i=0;i<NUMBOX;i++)
		{
		// Add the colour
    	VIFDynamicDMA.Add128(PACKED_RGBA(	Box[i].r,
    										Box[i].g,
    										Box[i].b,
    										0x80));
    	    	
    	// Add the position   	   	
    	VIFDynamicDMA.Add128(PACKED_XYZ2(	(2048 + Box[i].x)<< 4, 
    										(2048 + Box[i].y)<< 4, 
    										Box[i].z, 
    										0));
    	
    	// Add the colour
    	VIFDynamicDMA.Add128(PACKED_RGBA(	Box[i].r,
    										Box[i].g,
    										Box[i].b,
    										0x80));
    	    	
    	// Add the position   	   	
    	VIFDynamicDMA.Add128(PACKED_XYZ2(	(2048 + Box[i].x + Box[i].w)<< 4, 
    										(2048 + Box[i].y - Box[i].h)<< 4, 
    										Box[i].z, 
    										0));
    	  	
     	}
	
	// End the direct mode
	VIFDynamicDMA.EndDirect();
	
}


// Configure the box parameters
void ConfigureBox(Ps2Box_t * B, int x, int y, int z, int w, int h, 
								u_char r, u_char g, u_char b)
{
	B->x = x;
	B->y = y;
	B->z = z;
	B->w = w;
	B->h = h;
	B->r = r;
	B->g = g;
	B->b = b;
}

// Perform some game Logic
void GameLogic(void)
{
	int factor = 2;
	
	if (pad[0].buttons & PAD_CROSS)factor = 8;
	
	if (pad[0].buttons & PAD_UP)	Box[0].y -= factor;
	if (pad[0].buttons & PAD_DOWN)	Box[0].y += factor;
	if (pad[0].buttons & PAD_LEFT)	Box[0].x -= factor;
	if (pad[0].buttons & PAD_RIGHT)	Box[0].x += factor;
}